import math
from copy import copy
from polysurface import spline_double_divide, Offset, SurfaceFromCurves
from car_util import * # yeahhh don't do this
from pyopenscad import * # or this.  *sigh*.
from roof import roof_rear_pts, Roof
from bonnet import Bonnet
from front_section import Front
from floor import Floor
from airbox import Airbox
from backbox import BackBox
from rearsection import RearSection
from door_util import (calc_door_offsets, pmid, doorlinepts, doorsillpts,
                       make_points)
from nutsandbolts import M6SmallHeadHex, M4PoziHead, M5PoziHead, M3PoziHead


###############
# car
###############

# helps implicitcad (no support for color)
def _color(c):
    return union()

def create_car(t, _scl, _mirror, colour):

    dthin_wall = 0.0015 # 1.5 mm

    exploded = -0.20*_scl
    if exploded != 0:
        htadjust = _scl
    else:
        htadjust = 0

    bwid, bht  = _scl*0.0078, _scl*0.017

    # connector
    c = color([0.8, 1.0, 0.8, 0.6])
    p = union()
    g = difference()
    woff = 0.0
    doff = 0.0
    # main body
    d = cylinder(r=_scl*0.0150, h=_scl*0.007, segments=30, center=True)
    d = translate([0, 0, -_scl*0.0005]).add(d)
    g.add(d)
    # remove centre slide-rail
    d = cube([_scl*0.0085, _scl*0.022, _scl*0.009], center=True)
    g.add(translate([0,_scl*0.0045, -_scl*0.000]).add(d))
    p.add(g)
    # hammer cylinder
    d = cylinder(r=_scl*0.0003+bwid, h=_scl*0.020, segments=30, center=True)
    g.add(d)
    # fill back in offset parts for locking t-nut
    d = union()
    r = _scl*0.01
    for w in [-1.0, 1.0]:
        _d = cube([_scl*0.0050, _scl*0.0037, _scl*0.006], center=True)
        d.add(translate([_scl*0.00675*w, _scl*0.0060*w, -_scl*0.001]).add(_d))
        # and the base bit
        _d = cube([_scl*0.0060, _scl*0.0160, _scl*0.003], center=True)
        d.add(translate([(bwid-0.00055*_scl)*w, 0, _scl*0.0015]).add(_d))
    d.add(p)
    # fill in right at back, strengthen
    _d = difference()
    b = cube([_scl*0.0160, _scl*0.0062, _scl*0.003], center=True)
    _d.add(translate([0, -_scl*0.00675, _scl*0.0015]).add(b))
    b = cylinder(r=bwid/2*0.707+_scl*0.003, h=_scl*0.0100, segments=30, center=True)
    _d.add(b)
    d.add(_d)
    p = difference()
    p.add(d)
    for w in [-1.0, 1.0]:
        _d = M5PoziHead(6.0, add_nut=False, countersunk=True,
                        extra_diam=0.25).obj()
        p.add(translate([_scl*0.00795*w, _scl*0.00735*w, -_scl*0.0019]).add(_d))
        _d = M3PoziHead(6.0, add_nut=False, countersunk=True,
                        extra_diam=0.25).obj()
        p.add(translate([-_scl*0.00795*w, _scl*0.00735*w, -_scl*0.0019]).add(_d))
    # offset
    p = translate([_scl*woff, _scl*doff, _scl*-0.0070]).add(p)
    c.add(translate([-exploded*0.2, 0, htadjust*0.01299]).add(p))
    t.extras.add(c)

    # t-nut
    c = color([0.8, 1.0, 0.8, 0.6])
    p = union()
    g = difference()
    woff = 0.0
    doff = 0.0
    # main body
    b = difference()
    d = cylinder(r=bwid, h=_scl*0.0035, segments=30, center=True)
    b.add(d)
    _d = cube([bwid, bht, _scl*0.004], center=True)
    d = translate([bwid,0,0]).add(_d)
    b.add(d)
    d = translate([-bwid,0,0]).add(_d)
    b.add(d)
    g.add(b)
    # grippers
    sep = 0.00115
    for i in range(4):
        d = cylinder(r=_scl*0.00045, h=_scl*0.019, segments=30, center=True)
        d = rotate([90, 0, 0]).add(d)
        g.add(translate([(i+0.5)*_scl*sep, 0, _scl*0.0020]).add(d))
        g.add(translate([-(i+0.5)*_scl*sep, 0, _scl*0.0020]).add(d))
    p.add(g)
    # offset parts for locking hammerhead
    r = _scl*0.01
    lr = bwid/2
    for w in [lr/2, lr/2]:
        d = cube([lr, lr, _scl*0.005], center=True)
        p.add(translate([w, w, _scl*0.001]).add(d))
        p.add(translate([-w, -w, _scl*0.0010]).add(d))
    # hammer cylinder
    d = cylinder(r=lr, h=_scl*0.005, segments=30, center=True)
    d = translate([0, 0, _scl*0.001]).add(d)
    p.add(d)
    # remove centre
    d = difference()
    d.add(p)
    #p = d.add(cylinder(r=_scl*0.0032, h=_scl*0.02, segments=30, center=True))
    x = thread_for_nut(diameter=6, length=5.27, usrclearance=0.1)
    x = translate([0, 0, -_scl*0.00176]).add(x)
    p = d.add(x)
    # offset
    #p = rotate([0, 0, 90]).add(p)
    p = translate([_scl*woff, _scl*doff, _scl*-0.009]).add(p)
    c.add(translate([-exploded*0.1, 0, htadjust*0.015490]).add(p))
    t.extras.add(c)

    # demo pipe
    c = color([0.4, 0.7, 0.8, 0.4])
    t.extras.add(c)
    p = difference()
    d = cylinder(r=_scl*dthin/2, h=_scl*0.03, segments=30)
    p.add(d)
    d = cylinder(r=_scl*(dthin/2-dthin_wall), h=scl*1.01, segments=30)
    d = translate([0, 0, _scl*-0.001]).add(d)
    p.add(d)
    c.add(translate([1*exploded*0.1, 0.00, -htadjust*0.01701]).add(p))

    # fitting, inner
    c = color([0.9, 0.3, 0.8, 0.6])
    t.extras.add(c)
    p = difference()
    d = cylinder(r=_scl*(dthin/2.-dthin_wall-0.0001), h=_scl*0.02, segments=30)
    p.add(d)
    d = cylinder(r=_scl*0.0066, h=_scl*0.30, segments=6)
    d = rotate([0, 0, 45]).add(d) # for slot into outer fitting
    d = translate([0, 0.00, -_scl*0.001]).add(d)
    p.add(d)
    # remove slot parts
    lr = _scl*0.0112
    d = cube([lr*2, lr, _scl*0.008], center=True)
    _d = rotate([0, 0, 45]).add(d) # for slot into outer fitting
    d = translate([0, 0, -_scl*0.001]).add(_d)
    p.add(d)
    _d = rotate([0, 0, 90]).add(_d)
    d = translate([scl*0.005, scl*0.005, -_scl*0.001]).add(_d)
    p.add(d)
    d = translate([-scl*0.005, -scl*0.005, -_scl*0.001]).add(_d)
    p.add(d)
    p = translate([0, 0, _scl*-0.0039]).add(p)
    c.add(translate([2*exploded*0.1, 0.01, -htadjust*0.00701]).add(p))

    # fitting, outer
    c = color([0.9, 0.7, 0.8, 0.3])
    t.extras.add(c)
    p = difference()
    d = cylinder(r=_scl*(dthin/2.+dthin_wall), h=_scl*0.02, segments=30)
    p.add(d)
    d = cylinder(r=_scl*(dthin/2.+0.0002), h=_scl*0.05, segments=30)
    d = translate([0, 0.00, _scl*0.0030]).add(d)
    p.add(d)
    # offset parts for locking hammerhead
    r = _scl*0.01
    lr = bwid/2+_scl*0.0003
    for w in [lr/2, lr/2]:
        d = cube([lr, lr, _scl*0.008], center=True)
        p.add(translate([w, w, 0]).add(d))
        p.add(translate([-w, -w, 0]).add(d))
    # hammer cylinder
    d = cylinder(r=lr, h=_scl*0.008, segments=30, center=True)
    p.add(d)
    _p = p
    p = difference()
    d = cylinder(r=_scl*(dthin/2.-dthin_wall+0.0001), h=_scl*0.06,
                segments=30, center=True)
    p.add(d)
    # remove slot parts
    lr = _scl*0.0111
    d = cube([lr*2, lr, _scl*0.008], center=True)
    _d = rotate([0, 0, 45]).add(d) # for slot into outer fitting
    d = translate([0, 0, -_scl*0.001]).add(_d)
    p.add(d)
    _d = rotate([0, 0, 90]).add(_d)
    d = translate([scl*0.005, scl*0.005, -_scl*0.001]).add(_d)
    p.add(d)
    d = translate([-scl*0.005, -scl*0.005, -_scl*0.001]).add(_d)
    p.add(d)

    p = _p.add(p)
    
    # drop down
    p = translate([0, 0.00, -_scl*0.004]).add(p)
    c.add(translate([3*exploded*0.1, 0.01, -htadjust*0.00701]).add(p))

    # bolt
    c = color([0.4, 0.2, 0.4, 0.6])
    #bolt = M6SmallHeadHex(20.0, add_nut=False).obj()
    bolt = thread_for_screw(diameter=6, length=10.0)
    bolt = rotate([0, 0, 20]).add(bolt)
    bolt = translate([0, 0, -_scl*0.0060]).add(bolt)
    h = cylinder(r=6.4, h=4, segments=6)
    h = rotate([0, 0, 45]).add(h)
    h = translate([0, 0, _scl*0.0100-0.01]).add(h)
    bolt.add(h)

    #bolt = rotate([0, 180, 0]).add(bolt)
    c.add(translate([0, _scl*0.00, -_scl*0.0049]).add(bolt))
    t.extras.add(c)
    

if __name__ == '__main__':

    # get the threads library
    use("threads-library-by-cuiso-v1.scad")

    scl = 1000.0
    t = Model()
    c = [0.6, 0.5, 0.8, 1.0]
    c = True
    create_car(t, scl, 1.0, c)

    _m = t.get_model()
    #_m = rotate([0, 0, -45]).add(_m)
    _m = rotate([0, 0, 0]).add(_m)

    if False:
        m = difference()
        m.add(_m)
        c = cube([scl*1.0, scl*1.0, scl*1.0])
        c = translate([0, -scl*0.5, -scl*0.5]).add(c)
        m.add(c)
    else:
        m = _m
    print m

    scad_render_to_file(m, "t_join_prog2.scad")
    f = open("pipe_t_join.scad", "w")
    f.write("""include<t_join_prog2.scad>;
//%import_stl(filename = "car_model.stl");
    """)
    f.close()

    print "num joints:", len(t.joints)
    print "num pipes:", len(t.pipes)
    print "pipe len (m):", t.plen
